==> liquidwar6-0.0.1beta.tar.gz: liquidwar6-0.0.1beta/src/lib/gfx/mod-gl/gl-menu/gl-cylinder/gl-cylinder-display.c <==
/*
  Liquid War 6 is a unique multiplayer wargame.
  Copyright (C)  2005, 2006  Christian Mauduit <ufoot@ufoot.org>

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

  Liquid War 6 homepage : http://www.gnu.org/software/liquidwar6/
  Contact author        : ufoot@ufoot.org
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <GL/glu.h>

#include "config.h"
#include "../../mod-gl.h"
#include "gl-cylinder-internal.h"

#define TEST_MENU_SIZE 8

/*
 * OpenGL wizardry, to prepare view parameters.
 */
static void
prepare_view (MOD_GL_UTILS_CONTEXT * utils_context,
	      _MOD_GL_MENU_CYLINDER_CONTEXT * cylinder_context)
{
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
}

static void
draw_cylinder (MOD_GL_UTILS_CONTEXT * utils_context,
	       _MOD_GL_MENU_CYLINDER_CONTEXT * cylinder_context,
	       int i, int n, float ratio)
{
  _mod_gl_menu_cylinder_draw_cylinder (utils_context, cylinder_context,
				       GL_RENDER, i, n, ratio);
}

static SDL_Color
get_fg_color (MOD_GL_UTILS_CONTEXT * utils_context,
	      _MOD_GL_MENU_CYLINDER_CONTEXT * cylinder_context, int select)
{
  SDL_Color color;

  if (select)
    {
      color = mod_gl_utils_color_sys2sdl (&(utils_context->const_data.
					    menu_fg_selected));
    }
  else
    {
      color =
	mod_gl_utils_color_sys2sdl (&(utils_context->const_data.menu_fg));
    }

  return color;
}

static SDL_Color
get_bg_color (MOD_GL_UTILS_CONTEXT * utils_context,
	      _MOD_GL_MENU_CYLINDER_CONTEXT * cylinder_context, int select)
{
  SDL_Color color;

  if (select)
    {
      color = mod_gl_utils_color_sys2sdl (&(utils_context->const_data.
					    menu_bg_selected));
    }
  else
    {
      color =
	mod_gl_utils_color_sys2sdl (&(utils_context->const_data.menu_bg));
    }

  return color;
}

/*
 * Draw a basic text menu item (button).
 */
static void
draw_button (MOD_GL_UTILS_CONTEXT * utils_context,
	     _MOD_GL_MENU_CYLINDER_CONTEXT * cylinder_context, char *text,
	     int select, int i, int n)
{
  SDL_Surface *text_surface = NULL;
  SDL_Surface *outline_surface = NULL;
  GLuint text_texture;
  char *utf8;
  SDL_Color bg_color;
  SDL_Color fg_color;
  int ret = 0;
  SDL_Rect text_rect;
  SDL_Rect outline_rect;

  outline_surface =
    mod_gl_utils_get_button_from_menucache (utils_context, text, select);

  if (outline_surface == NULL)
    {
      utf8 = lw6sys_locale_to_utf8 (text);
      if (utf8 != NULL)
	{
	  fg_color = get_fg_color (utils_context, cylinder_context, select);
	  bg_color = get_bg_color (utils_context, cylinder_context, select);
	  text_surface =
	    TTF_RenderUTF8_Shaded (utils_context->font_data.menu, utf8,
				   fg_color, bg_color);
	  if (text_surface != NULL)
	    {
	      utils_context->surface_counter.new_counter++;
	      outline_surface =
		mod_gl_utils_create_surface (utils_context,
					     text_surface->w +
					     2 *
					     utils_context->const_data.
					     menu_font_size,
					     MOD_GL_UTILS_MENU_TEXTURE_H);
	      if (outline_surface != NULL)
		{
		  text_rect.x = 0;
		  text_rect.y = 0;
		  text_rect.w = text_surface->w;
		  text_rect.h = text_surface->h;
		  outline_rect.x = (outline_surface->w - text_surface->w) / 2;
		  outline_rect.y = (outline_surface->h - text_surface->h) / 2;
		  outline_rect.w = outline_surface->w;
		  outline_rect.h = outline_surface->h;

		  mod_gl_utils_clear_surface_with_color (outline_surface,
							 bg_color);
		  SDL_BlitSurface (text_surface, &text_rect, outline_surface,
				   &outline_rect);

		  if (!mod_gl_utils_store_button_in_menucache
		      (utils_context, text, select, outline_surface))
		    {
		      /*
		       * Storing stuff in menucache should always work, but in
		       * case it wouldn't work, we just cancel everything,
		       * it's better not to display something just one time
		       * than risk a memory leak. Besides doing this will
		       * show the user that "something" is going wrong, which
		       * is an order of magnitude better than an obscure invisible
		       * memory leak...
		       */
		      mod_gl_utils_delete_surface (utils_context,
						   outline_surface);
		      outline_surface = NULL;
		    }
		}
	      mod_gl_utils_delete_surface (utils_context, text_surface);
	    }
	  LW6SYS_FREE (utf8);
	}
    }

  if (outline_surface != NULL)
    {
      text_texture =
	mod_gl_utils_surface2texture_xywh_using_cache (utils_context,
						       outline_surface,
						       (MOD_GL_UTILS_MENU_TEXTURE_W
							-
							outline_surface->w) /
						       2,
						       (MOD_GL_UTILS_MENU_TEXTURE_H
							-
							outline_surface->h) /
						       2,
						       MOD_GL_UTILS_MENU_TEXTURE_W,
						       MOD_GL_UTILS_MENU_TEXTURE_H,
						       text, select);
      if (text_texture)
	{
	  glEnable (GL_TEXTURE_2D);
	  glColor3f (1.0f, 1.0f, 0.0f);
	  glBindTexture (GL_TEXTURE_2D, text_texture);

	  glMatrixMode (GL_TEXTURE);
	  glPushMatrix ();
	  glLoadIdentity ();
	  glScalef (1.0f, -1.0f, 1.0f);
	  glTranslatef (0.0f,
			-0.25f + cylinder_context->const_data.rotate_offset,
			0.0f);
	  glRotatef (-90.0f, 0.0f, 0.0f, 1.0f);

	  draw_cylinder (utils_context, cylinder_context, i, n,
			 ((float) MOD_GL_UTILS_MENU_TEXTURE_W) /
			 ((float) outline_surface->w));

	  glMatrixMode (GL_TEXTURE);
	  glPopMatrix ();

	  ret = 1;

	  // _mod_gl_schedule_delete_texture(context,text_texture);
	}
    }

  /*
   * Note: it's important *NOT* to delete outline_surface.
   * Indeed it's kept in the menucache structure. It will
   * be free when needed, wizardry inside menucache.c 8-)
   */

  if (!ret)
    {
      lw6sys_log (LW6SYS_LOG_WARNING, "gfx-mod-gl",
		  _("unable to draw button \"%s\""), text);
    }
}

/*
 * Display a menu.
 */
void
_mod_gl_menu_cylinder_display (MOD_GL_UTILS_CONTEXT * utils_context,
			       _MOD_GL_MENU_CYLINDER_CONTEXT *
			       cylinder_context, LW6GFX_MENU * menu)
{
  int i;
  LW6GFX_MENUITEM *menuitem;
  LW6GFX_MENU_STATE *menu_state;

  mod_gl_utils_set_render_mode_3d_menu (utils_context);

  prepare_view (utils_context, cylinder_context);

  if (lw6sys_assoc_has_key (cylinder_context->menu_states, menu->id))
    {
      menu_state =
	(LW6GFX_MENU_STATE *) lw6sys_assoc_get (cylinder_context->menu_states,
						menu->id);
    }
  else
    {
      menu_state = lw6gfx_menu_state_new ();
      lw6sys_assoc_set (&(cylinder_context->menu_states), menu->id,
			(void *) menu_state);
    }
  lw6gfx_menu_state_update (menu_state, menu,
			    utils_context->const_data.
			    menu_max_displayed_items);

  for (i = 0; i < menu_state->nb_items_displayed; ++i)
    {
      menuitem = menu->items[i + menu_state->first_item_displayed];
      switch (menuitem->type)
	{
	case LW6GFX_MENUITEM_NONE:
	  break;
	case LW6GFX_MENUITEM_BUTTON:
	  draw_button (utils_context,
		       cylinder_context,
		       menuitem->label,
		       i + menu_state->first_item_displayed ==
		       menu->selected_item, i,
		       menu_state->nb_items_displayed);
	  break;
	case LW6GFX_MENUITEM_EDIT:
	  break;
	case LW6GFX_MENUITEM_SLIDER:
	  break;
	default:
	  lw6sys_log (LW6SYS_LOG_WARNING, "gfx-mod-gl",
		      _("unknown menuitem type %d"), menuitem->type);
	}
    }
}

void
mod_gl_menu_cylinder_display (MOD_GL_UTILS_CONTEXT * utils_context,
			      void *cylinder_context, LW6GFX_MENU * menu)
{
  _mod_gl_menu_cylinder_display (utils_context,
				 (_MOD_GL_MENU_CYLINDER_CONTEXT *)
				 cylinder_context, menu);
}

==> liquidwar6-0.0.2beta.tar.gz: liquidwar6-0.0.2beta/src/lib/gfx/mod-gl/gl-menu/gl-cylinder/gl-cylinder-display.c <==
/*
  Liquid War 6 is a unique multiplayer wargame.
  Copyright (C)  2005, 2006, 2007  Christian Mauduit <ufoot@ufoot.org>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  

  Liquid War 6 homepage : http://www.gnu.org/software/liquidwar6/
  Contact author        : ufoot@ufoot.org
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <GL/glu.h>

#include "config.h"
#include "../../mod-gl.h"
#include "gl-cylinder-internal.h"

#define TEST_MENU_SIZE 8

/*
 * OpenGL wizardry, to prepare view parameters.
 */
static void
prepare_view (MOD_GL_UTILS_CONTEXT * utils_context,
	      _MOD_GL_MENU_CYLINDER_CONTEXT * cylinder_context)
{
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
}

static void
draw_cylinder (MOD_GL_UTILS_CONTEXT * utils_context,
	       _MOD_GL_MENU_CYLINDER_CONTEXT * cylinder_context,
	       int i, int n, float ratio)
{
  _mod_gl_menu_cylinder_draw_cylinder (utils_context, cylinder_context,
				       GL_RENDER, i, n, ratio);
}

static SDL_Color
get_fg_color (MOD_GL_UTILS_CONTEXT * utils_context,
	      _MOD_GL_MENU_CYLINDER_CONTEXT * cylinder_context, int select)
{
  SDL_Color color;

  if (select)
    {
      color = mod_gl_utils_color_sys2sdl (&(utils_context->const_data.
					    menu_fg_selected));
    }
  else
    {
      color =
	mod_gl_utils_color_sys2sdl (&(utils_context->const_data.menu_fg));
    }

  return color;
}

static SDL_Color
get_bg_color (MOD_GL_UTILS_CONTEXT * utils_context,
	      _MOD_GL_MENU_CYLINDER_CONTEXT * cylinder_context, int select)
{
  SDL_Color color;

  if (select)
    {
      color = mod_gl_utils_color_sys2sdl (&(utils_context->const_data.
					    menu_bg_selected));
    }
  else
    {
      color =
	mod_gl_utils_color_sys2sdl (&(utils_context->const_data.menu_bg));
    }

  return color;
}

/*
 * Draw a basic text menu item (button).
 */
static void
draw_button (MOD_GL_UTILS_CONTEXT * utils_context,
	     _MOD_GL_MENU_CYLINDER_CONTEXT * cylinder_context, char *text,
	     int select, int i, int n)
{
  SDL_Surface *text_surface = NULL;
  SDL_Surface *outline_surface = NULL;
  GLuint text_texture;
  char *utf8;
  SDL_Color bg_color;
  SDL_Color fg_color;
  int ret = 0;
  SDL_Rect text_rect;
  SDL_Rect outline_rect;

  outline_surface =
    mod_gl_utils_get_button_from_menucache (utils_context, text, select);

  if (outline_surface == NULL)
    {
      utf8 = lw6sys_locale_to_utf8 (text);
      if (utf8 != NULL)
	{
	  fg_color = get_fg_color (utils_context, cylinder_context, select);
	  bg_color = get_bg_color (utils_context, cylinder_context, select);
	  text_surface =
	    TTF_RenderUTF8_Shaded (utils_context->font_data.menu, utf8,
				   fg_color, bg_color);
	  if (text_surface != NULL)
	    {
	      utils_context->surface_counter.new_counter++;
	      outline_surface =
		mod_gl_utils_create_surface (utils_context,
					     text_surface->w +
					     2 *
					     utils_context->const_data.
					     menu_font_size,
					     MOD_GL_UTILS_MENU_TEXTURE_H);
	      if (outline_surface != NULL)
		{
		  text_rect.x = 0;
		  text_rect.y = 0;
		  text_rect.w = text_surface->w;
		  text_rect.h = text_surface->h;
		  outline_rect.x = (outline_surface->w - text_surface->w) / 2;
		  outline_rect.y = (outline_surface->h - text_surface->h) / 2;
		  outline_rect.w = outline_surface->w;
		  outline_rect.h = outline_surface->h;

		  mod_gl_utils_clear_surface_with_color (outline_surface,
							 bg_color);
		  SDL_BlitSurface (text_surface, &text_rect, outline_surface,
				   &outline_rect);

		  if (!mod_gl_utils_store_button_in_menucache
		      (utils_context, text, select, outline_surface))
		    {
		      /*
		       * Storing stuff in menucache should always work, but in
		       * case it wouldn't work, we just cancel everything,
		       * it's better not to display something just one time
		       * than risk a memory leak. Besides doing this will
		       * show the user that "something" is going wrong, which
		       * is an order of magnitude better than an obscure invisible
		       * memory leak...
		       */
		      mod_gl_utils_delete_surface (utils_context,
						   outline_surface);
		      outline_surface = NULL;
		    }
		}
	      mod_gl_utils_delete_surface (utils_context, text_surface);
	    }
	  LW6SYS_FREE (utf8);
	}
    }

  if (outline_surface != NULL)
    {
      text_texture =
	mod_gl_utils_surface2texture_xywh_using_cache (utils_context,
						       outline_surface,
						       (MOD_GL_UTILS_MENU_TEXTURE_W
							-
							outline_surface->w) /
						       2,
						       (MOD_GL_UTILS_MENU_TEXTURE_H
							-
							outline_surface->h) /
						       2,
						       MOD_GL_UTILS_MENU_TEXTURE_W,
						       MOD_GL_UTILS_MENU_TEXTURE_H,
						       text, select);
      if (text_texture)
	{
	  glEnable (GL_TEXTURE_2D);
	  glColor3f (1.0f, 1.0f, 0.0f);
	  glBindTexture (GL_TEXTURE_2D, text_texture);

	  glMatrixMode (GL_TEXTURE);
	  glPushMatrix ();
	  glLoadIdentity ();
	  glScalef (1.0f, -1.0f, 1.0f);
	  glTranslatef (0.0f,
			-0.25f + cylinder_context->const_data.rotate_offset,
			0.0f);
	  glRotatef (-90.0f, 0.0f, 0.0f, 1.0f);

	  draw_cylinder (utils_context, cylinder_context, i, n,
			 ((float) MOD_GL_UTILS_MENU_TEXTURE_W) /
			 ((float) outline_surface->w));

	  glMatrixMode (GL_TEXTURE);
	  glPopMatrix ();

	  ret = 1;

	  // _mod_gl_schedule_delete_texture(context,text_texture);
	}
    }

  /*
   * Note: it's important *NOT* to delete outline_surface.
   * Indeed it's kept in the menucache structure. It will
   * be free when needed, wizardry inside menucache.c 8-)
   */

  if (!ret)
    {
      lw6sys_log (LW6SYS_LOG_WARNING, "gfx-mod-gl",
		  _("unable to draw button \"%s\""), text);
    }
}

/*
 * Display a menu.
 */
void
_mod_gl_menu_cylinder_display (MOD_GL_UTILS_CONTEXT * utils_context,
			       _MOD_GL_MENU_CYLINDER_CONTEXT *
			       cylinder_context, LW6GFX_MENU * menu)
{
  int i;
  LW6GFX_MENUITEM *menuitem;
  LW6GFX_MENU_STATE *menu_state;

  mod_gl_utils_set_render_mode_3d_menu (utils_context);

  prepare_view (utils_context, cylinder_context);

  if (lw6sys_assoc_has_key (cylinder_context->menu_states, menu->id))
    {
      menu_state =
	(LW6GFX_MENU_STATE *) lw6sys_assoc_get (cylinder_context->menu_states,
						menu->id);
    }
  else
    {
      menu_state = lw6gfx_menu_state_new ();
      lw6sys_assoc_set (&(cylinder_context->menu_states), menu->id,
			(void *) menu_state);
    }
  lw6gfx_menu_state_update (menu_state, menu,
			    utils_context->const_data.
			    menu_max_displayed_items);

  for (i = 0; i < menu_state->nb_items_displayed; ++i)
    {
      menuitem = menu->items[i + menu_state->first_item_displayed];
      switch (menuitem->type)
	{
	case LW6GFX_MENUITEM_NONE:
	  break;
	case LW6GFX_MENUITEM_BUTTON:
	  draw_button (utils_context,
		       cylinder_context,
		       menuitem->label,
		       i + menu_state->first_item_displayed ==
		       menu->selected_item, i,
		       menu_state->nb_items_displayed);
	  break;
	case LW6GFX_MENUITEM_EDIT:
	  break;
	case LW6GFX_MENUITEM_SLIDER:
	  break;
	default:
	  lw6sys_log (LW6SYS_LOG_WARNING, "gfx-mod-gl",
		      _("unknown menuitem type %d"), menuitem->type);
	}
    }
}

void
mod_gl_menu_cylinder_display (MOD_GL_UTILS_CONTEXT * utils_context,
			      void *cylinder_context, LW6GFX_MENU * menu)
{
  _mod_gl_menu_cylinder_display (utils_context,
				 (_MOD_GL_MENU_CYLINDER_CONTEXT *)
				 cylinder_context, menu);
}
